<?php

/**
 * Copyright (c) 2013, Matt Ferris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @copyright Copyright (c) 2013 Matt Ferris
 * @author Matt Ferris <matt@bueller.ca>
 */

namespace ca\bueller\Formation;

class FormField implements iFormField
{
    /**
     * @var string
     */
    protected $name;

    /**
     * @var mixed
     */
    protected $value;

    /**
     * @var iValidator
     */
    protected $validator;

    /**
     * @var bool
     */
    protected $required = false;


    /**
     * @param string $name
     * @param bool $required
     * @param iValidator $validator
     * @return void
     */
    public function __construct ( $name, $required = false, iValidator $validator = NULL )
    {
        if (empty($name))
        {
            throw new \InvalidArgumentException('$name expects non-empty string');
        }

        if (!is_bool($required))
        {
            throw new \InvalidArgumentException('$required expects bool');
        }

        $this->name = $name;
        $this->validator = $validator;
        $this->required = $required;
    }


    /**
     * @return string
     */
    public function getName ()
    {
        return $this->name;
    }


    /**
     * @return mixed
     */
    public function getValue ()
    {
        return $this->value;
    }


    /**
     * @param mixed $value
     * @return void
     */
    public function setValue ( $value )
    {
        $this->value = $value;
    }


    /**
     * @return bool
     */
    public function isRequired ()
    {
        return $this->required;
    }


    /**
     * @return bool
     */
    public function isValid ()
    {
        $isValid = true;

        if ($this->validator !== NULL)
        {
            $isValid = $this->validator->validate($this->value);
        }

        return $isValid;
    }
}
